// Beta nested model 
#include <TMB.hpp>

template<class Type>
Type objective_function<Type>::operator() ()
{
  DATA_VECTOR(y);         // Observations
  DATA_SPARSE_MATRIX(X);  // Fixed effect design matrix
  PARAMETER_VECTOR(beta); // Fixed effects vector
  DATA_SPARSE_MATRIX(Z);  // Random effect design matrix (ui)
  DATA_SPARSE_MATRIX(W);  // Random effect design matrix (uij)
  PARAMETER_VECTOR(ui);   // Random effects vector
  PARAMETER_VECTOR(uij);  // Random effects vector
  PARAMETER(logtau1);     // Random effect standard deviations (ui)
  PARAMETER(logtau2);     // Random effect standard deviations (uij)
  PARAMETER(logphi);      // Precision parameter (Beta)

  // Parametrization
  Type tau1 = exp(logtau1); 
  Type tau2 = exp(logtau2); 
  Type phi = exp(logphi); 
  
  // Distribution of random effect (u):
  Type nll = 0;
  nll -= sum(dnorm(ui, Type(0), tau1, true)) + sum(dnorm(uij, Type(0), tau2, true)); 

  // Distribution of obs given random effects (y|u):
    vector<Type> mu = exp(X * beta + Z * ui + W * uij)/(1 + exp(X * beta + Z * ui + W * uij));
    
  // Log-likelihood 
    for(int i=0; i<y.size(); i++)
    nll -= dbeta(y(i), mu(i)*phi, (1 - mu(i))*phi, true);

  // Report
  ADREPORT(phi);
  ADREPORT(tau1);
  ADREPORT(tau2);
  return nll;
}

